/**
 * Copyright (C) 2012 Matteo Tamigi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA.
 */
 
#include "PDFSec.hpp"

//  Definizioni di tutte le funzioni pubbliche della classe PDFSec:
//  -Costruttori e distruttori
//  -Metodi SET e GET
//  -Metodi che permettono l'output (PrintInfos)
//  -Funzioni pubbliche che richiamano altre funzioni private (LoadPdfInfos)
 
PDFSec :: PDFSec () : nomeFile(), is(),
                      userPswd() {    //Variabili relative al cracking
    //Azzero tutti i puntatori
    infoPdf = NULL;
    state = NULL;
    charSet = NULL;
    key = NULL;
    u_string = NULL;
    cifr = NULL;
    tmp = NULL;
    tmpkey = NULL;
    digest = NULL;
    
    clearPdfInfos ();   //Pulisco la memoria per la lettura del PDF
    clearPassState();   //Pulisco la memoria per la generazione delle user_password
    //DEBUG INFOS
    verbose = false;
}

PDFSec :: ~PDFSec () {
    clearPdfInfos();
    clearCracking();
    clearPassState();
}


void PDFSec :: PrintInfos () {
    uint8_t ch;
    if ( infoPdf ) {
        cout << "File name: " << nomeFile << " dimension: " << dimFile << " byte" << endl;
        cout << "PDF version "<< infoPdf->version_major << "." << infoPdf->version_minor << endl;
        
        if(ERRNOENCINFO != err ) {
            cout << "Security Handler: " << infoPdf->s_handler;
            cout << " version " << infoPdf->version << "." << infoPdf->revision << " length: " << infoPdf->length << endl;
            cout << "Permessi: " << infoPdf->permissions << endl;
            cout << "Encrypted Metadata: " << infoPdf->encryptMetaData << endl;
            cout << "FileID: ";
            for(unsigned i = 0; i < infoPdf->fileID.size(); i++) {
                ch = infoPdf->fileID[i];
                if(ch < 16)
                    printf("0%x", ch);
                else
                    printf("%x", ch);
            }
            cout << endl;
        
            cout << "user_string: ";
            for(unsigned i = 0; i < infoPdf->u_string.size(); i++) {
                ch = infoPdf->u_string[i];
                if(ch < 16)
	               printf("0%x", ch);
                else
	               printf("%x", ch);
            }
            cout << endl << "ownr_string: ";
            for(unsigned i = 0; i < infoPdf->o_string.size(); i++) {
                ch = infoPdf->o_string[i];
                if(ch < 16)
                    printf("0%x", ch);
                else
	                printf("%x", ch);
            }
            cout << endl;
        } else {
            cout << "No encryption infos!" << endl;
        }
    } else {
        cout << "No file loaded!" << endl;
    }
}

EncData * PDFSec :: GetPdfInfos () {
    //Genero una nuova istanza di EncData e la passo per riferimento
    EncData * e = new EncData;
    *e = *infoPdf;
    return e;
}

void PDFSec :: Benchmark () {
    time_t start, stop;
    uint8_t *buf = new uint8_t[32];
    uint8_t digest[16];
    unsigned i, j;
    //Effettuo i test di tutti gli algoritmi base
    cout << "Benchmark:\t\tAverage speed (calls / second):" << endl;
    start = clock();
    for(i = 0; i < 1000000; i++)
        md5(buf,32,digest);
    stop = clock();
    cout << "MD5:\t\t\t\t" << (int)((double)1000000/(stop-start)*1000) << endl;
    start = clock();
    for(i = 0; i < 100000; i++)
        md5_50(buf);
    stop = clock();
    cout << "MD5_50 (fast):\t\t\t" << (int)((double)1000000/(stop-start)*1000) << endl;
    start = clock();
    for(i = 0; i < 100000; i++)
        for(j = 0; j < 50; j++)
            md5(buf,16,buf);
    stop = clock();
    cout << "MD5_50 (slow):\t\t\t" << (int)((double)1000000/(stop-start)*1000) << endl << endl;
    
    start = clock();
    for(i = 0; i < 1000000; i++)
        Rc4(buf,32,digest,5,buf);
    stop = clock();
    cout << "RC4:\t\t\t\t" << (int)((double)1000000/(stop-start)*1000) << endl;
    start = clock();
    for(i = 0; i < 1000000; i++)
        Rc4_40(buf,32,digest,buf);
    stop = clock();
    cout << "RC4 fast (40):\t\t\t" << (int)((double)1000000/(stop-start)*1000) << endl;
    start = clock();
    for(i = 0; i < 1000000; i++)
        Rc4_128(buf,3,digest,buf);
    stop = clock();
    cout << "RC4 fast (128):\t\t\t" << (int)((double)1000000/(stop-start)*1000) << endl;
}
