/**
 * Copyright (C) 2012 Matteo Tamigi
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 
 * USA.
 */
 
#include <cstring>
#include <fstream>
#include <time.h>

#include "common.hpp"

#ifndef PDFSEC_HPP
#define PDFSEC_HPP

class PDFSec {
    public:
        //Funzioni pubbliche per la classe PDFSec (specificate in PDFSec.cpp)
        //Costruttori e distruttori
        PDFSec ();
        ~PDFSec ();
        //Funzioni per la shell da sostituire a parametri pubblici
        void PrintInfos ();
        void Benchmark ();
        //Funzioni primarie
        void LoadPdfInfos ( const string );
        //Funzioni specificate in PDFUserCrack.cpp
        void initCracking ( const string ); //Gli passo la user_password (se la conosco)
        //(min_password,max_password,charset,vettore di stato) default (1,5,STDCHARSET,NULL)
        string crackUserPassword (  unsigned , unsigned , const string , unsigned * );
        string crackOwnerPassword (  unsigned , unsigned , const string , unsigned * );
        //int crypt ( string , string , unsigned );
        //int decrypt ( string , string );
        //Metodi SET e GET
        void SetFile ( const string file ) { nomeFile = file; }
        string GetFile () { return nomeFile; }
        unsigned GetDimFile () { return dimFile; }
        int GetErr () { return err; }
        int GetWrn () { return wrn; }
        //DEBUG INFOS
        EncData * GetPdfInfos ();
        bool verbose;
    private:
        //VARIABILI
        //--------------------------------------------------------------------//
        string nomeFile;
        unsigned dimFile;
        //Variabili per il parsing e il mantenimento delle info sul file
        EncData *infoPdf;        
        ifstream is;
        int err;
        int wrn;
        //Variabili per il cracking del PDF
        bool isUserPswd;
        string userPswd;
        uint8_t * key;          //WorkSpace per la decifratura delle user_pswd
        uint8_t * key2;         //WorkSpace per la decifratura delle owner_pswd
        unsigned sizeKey;       //Dimensione del vettore key
        uint8_t * u_string;     //Copia della u_string
        uint8_t * o_string;     //Copia della u_string
        uint8_t * cifr;         //Chiave di testing per la revision 3
        unsigned length;
        uint8_t * tmp;          //Vettori temporanei
        uint8_t * tmpkey;
        uint8_t * digest;       //vettore di ritorno delle varie cifrature
        int a, b;             //Contatori standard
        //Variabili per la generazione veloce della key da cifrare
        uint8_t * charSet;      //Per l'ottimizzazione utilizzare una lista circolare
        unsigned lenSet;
        unsigned max_c , min_c;
        unsigned * state;       //Stato dell'avanzamento
        unsigned lenState;
        bool EOS;
        //--------------------------------------------------------------------//
        //METODI
        //--------------------------------------------------------------------//
        //Funzioni per la lettura del PDF (funzioni specificate in PDFParser.cpp)
        //Completate e (apparentemente) funzionanti! (rifatte da pdfcrack-0.11)
        int readPdfFile ();
        int findTrailer ();
        vector<uint8_t> parseID ( const string );
        bool getEncInfo ( const int );
        bool parseEncObject ( const string );
        vector<uint8_t> stringToByte ( const string );
        void clearPdfInfos ();
        //Funzioni per il cracking delle password del PDF (funzioni specificate in PDFCrack.cpp)
        //(rifatte da pdfcrack-0.11)
        void clearCracking();
        //Funzioni di cifratura e controllo delle user_password
        bool isUserPasswordV1R2 ();
        bool isUserPasswordV2R3 ();
        //Funzioni di cifratura e controllo della owner_password
        bool isOwnerPasswordV1R2 ();        //Versione 1 Revisione 2, con la user_password non nulla
        bool isOwnerPasswordV1R2_nuser ();  //Versione 1 Revisione 2, con la user_password nulla
        bool isOwnerPasswordV2R3 ();        //Versione 2 Revisione 3, con la user_password non nulla
        bool isOwnerPasswordV2R3_nuser ();  //Versione 2 Revisione 3, con la user_password nulla
        //Funzioni per la generazione della key da cifrare
        //Le funzioni lavorano direttamente sulla key (per ottimizzazione)
        //Al momento lavorano solo su di un vettore, il prossimo step  l'elaborazione
        //tramite thread di pi key
        void clearPassState ();
        void SetCharset ( const string );
        void SetState ( const unsigned , const unsigned , unsigned * );
        void SetNextPswd ();
        string GetCurrentPswd ();      //Estrae dallo stato e dalla key, la password corrente
        //--------------------------------------------------------------------//
};
 
 
 
 #endif
